package com.org.hktshop.service.encapsulation;

import com.org.hktshop.dto.ExcelColumnDto;
import com.org.hktshop.dto.VerifyCellDto;
import com.org.hktshop.entity.Page;
import com.org.hktshop.entity.encapsulation.FcProduct;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Set;

public interface FcProductService {
    void saveFcProduct(FcProduct fcProduct);

    void deleteFcProduct(String id);

    void updateFcProduct(FcProduct fcProduct);

    FcProduct getFcProduct(String id);

    Page<FcProduct> getFcProductPage(FcProduct fcProduct);

//    Page<FcProduct> getFcProductUserPage(FcProduct fcProduct);

    VerifyCellDto verifyImportData(MultipartFile excelFile);

    void importData(List<FcProduct> products);

    List assembleData(List<List<ExcelColumnDto>> rowList);

    Set<String> getBrandNames(String materialName, String packageName);

    List<FcProduct> getDefaultBom(String packageName);
}
